package freiburg;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Scanner;
import java.util.Stack;
import java.util.stream.Collectors;

class Freiburg {

    enum ProgTyp { PROGRAMM("P"), MENUE("M"), DIVERSE("D");
        private String code;
        ProgTyp(String code) { this.code = code; }
        public String getCode() { return code; }

        public static ProgTyp fromCode(String code) {
            for (ProgTyp progTyp : values()) {
                if (progTyp.code.equalsIgnoreCase(code))
                   return progTyp;
            }
            throw new IllegalArgumentException("Ungltiger Programmtyp: " + code);
        }
    }

    static class TEintrag {
        int M1, M2, M3, N1, N2, L1;
        ProgTyp progTyp;
        String BEZEICHNER, CODE;

        TEintrag(int M1, int M2, int M3, ProgTyp progTyp, String BEZEICHNER, String CODE, int N1, int N2, int L1) {
          this.M1 = M1; this.M2 = M2; this.M3 = M3;
          this.progTyp = progTyp; this.BEZEICHNER = BEZEICHNER; this.CODE = CODE;
          this.N1 = N1; this.N2 = N2; this.L1 = L1;
        }
        void kopiereVon(TEintrag quelle) {
          this.M1 = quelle.M1; this.M2 = quelle.M2; this.M3 = quelle.M3;
          this.progTyp = quelle.progTyp; this.BEZEICHNER = quelle.BEZEICHNER; this.CODE = quelle.CODE;
          this.N1 = quelle.N1; this.N2 = quelle.N2; this.L1 = quelle.L1;
        }
    }

    static class TMenueManager {
        private static final String HAUPTMENUE_BEZEICHNUNG = "Hauptmenue";
        
        List<TEintrag> gesamt = new ArrayList<>();
        Stack<TEintrag> mStack = new Stack<>();
        int m1 = 0, m2 = 0;
        String bezeichnung = HAUPTMENUE_BEZEICHNUNG;
        boolean menueAngezeigt = false;

        Optional<TEintrag> findeMenuepunkt(int m1, int m2, int m3) {
            return gesamt.stream()
                    .filter(eintrag -> eintrag.M1 == m1 && eintrag.M2 == m2 && eintrag.M3 == m3)
                    .findFirst();
        }

        boolean hatNachfolgeMenue(int n1, int n2) {
            return gesamt.stream().anyMatch(eintrag -> eintrag.M1 == n1 && eintrag.M2 == n2);
        }

        List<TEintrag> aktuelleMenuepunkte(int m1, int m2) {
            return gesamt.stream()
                    .filter(eintrag -> eintrag.M1 == m1 && eintrag.M2 == m2)
                    .collect(Collectors.toList());
        }

        void zeigeMenue() {
            List<TEintrag> menuepunkte = aktuelleMenuepunkte(m1, m2);
            if (menuepunkte.isEmpty()) {
                System.out.println("Fehler: Menuezeilen nicht vorhanden.");
            } else {
                System.out.println("\n" + bezeichnung + ":");
                for (TEintrag punkt : menuepunkte) {
                    System.out.println(String.format("%d: %s", punkt.M3, punkt.BEZEICHNER));
                    for (int i = 0; i < punkt.L1; i++) System.out.println();
                }
                System.out.println("0: Zurueck");
            }
        }

        boolean bearbeiteAuswahl(int auswahl) {
            if (auswahl == 0) {
                menueAngezeigt = false;
                if (mStack.isEmpty()) {
                    System.out.println("Programmende.");
                    return true;
                }
                TEintrag temp = mStack.pop();
                m1 = temp.M1; m2 = temp.M2;
                bezeichnung = temp.BEZEICHNER;
                return false;
            }

            Optional<TEintrag> optionalPunkt = findeMenuepunkt(m1, m2, auswahl);
            if (!optionalPunkt.isPresent()) {
                System.out.println("Ungueltige Auswahl. Keine Eintraege fuer das gewaehlte Menue.");
                return false;
            }
            
            TEintrag punkt = optionalPunkt.get();
            bezeichnung = punkt.BEZEICHNER;
            if (punkt.progTyp.equals(ProgTyp.PROGRAMM)) {
                System.out.println("Starte Programm: " + punkt.CODE);
            } else if (punkt.progTyp.equals(ProgTyp.MENUE)) {
                if (hatNachfolgeMenue(punkt.N1, punkt.N2)) {
                    mStack.push(punkt);
                    m1 = punkt.N1; m2 = punkt.N2;
                    menueAngezeigt = false;
                } else {
                    System.out.println("Ungueltige Auswahl. Aufgerufenes Menue nicht gefunden.");
                }
            } else {
                System.out.println("Fehler: Falscher Programmcode.");
            }
            return false;
        }

        void fuehreMenueAus() {
            try (Scanner scanner = new Scanner(System.in)) {
                while (true) {
                    if (!menueAngezeigt) {
                        zeigeMenue();
                        menueAngezeigt = true;
                    }
                    System.out.print("Auswahl: ");
                    try {
                        int auswahl = Integer.parseInt(scanner.nextLine());
                        if (bearbeiteAuswahl(auswahl))
                            break;
                    } catch (NumberFormatException e) {
                        System.out.println("Ungueltige Eingabe. Bitte geben Sie eine Zahl ein.");
                    }
                }
            }
        }

        void loadCSV(String fileName) {
            try (BufferedReader br = new BufferedReader(new FileReader(fileName))) {
                br.lines().skip(1).forEach(line -> {
                    try {
                        String[] parts = line.split(";");
                        ProgTyp progTyp = ProgTyp.fromCode(parts[3]); 
                        TEintrag eintrag = new TEintrag(
                                Integer.parseInt(parts[0]), Integer.parseInt(parts[1]), Integer.parseInt(parts[2]),
                                progTyp, parts[4], parts[5],
                                Integer.parseInt(parts[6]), Integer.parseInt(parts[7]), Integer.parseInt(parts[8]));
                        if (!progTyp.equals(ProgTyp.DIVERSE)) 
                            gesamt.add(eintrag);
                    } catch (Exception e) {
                        System.out.println("Fehler beim Einlesen der Zeile: " + line);
                    }
                });
            } catch (IOException e) {
                System.out.println("Fehler beim Lesen der Datei: " + e.getMessage());
            }
        }
    }

    public static void main(String[] args) {
        TMenueManager menueManager = new TMenueManager();
        menueManager.loadCSV("daten.csv");
        menueManager.fuehreMenueAus();
        System.out.println("Beenden");
    }
}
